local addonName = ...
local ldbi = LibStub("LibDBIcon-1.0")

local darkmoonProfessional_config
local darkmoonProfessional_init -- Have we initialized?
local darkmoonProfessional_minimap =
	{
		show = true,
	};


-- Register all of our widgets here so we can iterate over them
local Widgets = {} -- [frame] = cvar

-------------
-- Checkbox widget
-------------
local function checkboxGetCVar(self) return GetCVarBool(self.cvar) end
local function checkboxSetChecked(self) self:SetChecked(self:GetValue()) end
local function checkboxSetCVar(self, checked) addon:SetCVar(self.cvar, checked) end
local function checkboxOnClick(self)
	local checked = self:GetChecked()
    PlaySound(checked and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
	self:SetValue(checked)
end

local function checkboxDisable(self)
	self.label:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
end

local function checkboxEnable(self)
	self.label:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
end

local function newCheckbox(parent, cvar, getValue, setValue, label, description)
	local cvarTable = addon.hiddenOptions[cvar]
	if cvarTable then
		label = cvarTable['prettyName'] or cvar
		description = _G[cvarTable['description']] or cvarTable['description'] or 'No description'
	else
		label = label or '[PH] Label'
		description = description or '[PH] Description'
	end
	local check = CreateFrame("CheckButton", "AIOCheck" .. label, parent, "InterfaceOptionsCheckButtonTemplate")

	check.cvar = cvar
	check.GetValue = getValue or checkboxGetCVar
	check.SetValue = setValue or checkboxSetCVar
	check:SetScript('OnShow', checkboxSetChecked)
	check:SetScript("OnClick", checkboxOnClick)
	check.label = _G[check:GetName() .. "Text"]
	check.label:SetText(label)
	check.tooltipText = label
	check.tooltipRequirement = description

	check:HookScript('OnDisable', checkboxDisable)
	check:HookScript('OnEnable', checkboxEnable)

	Widgets[check] = cvar
	return check
end

--local DMPOptions = {};

--local function setupOptionsPanel()
--	DMPOptions.panel = CreateFrame( "Frame", "DMPOptions", UIParent );
--	DMPOptions.panel.name = "Darkmoon Professional";
--	InterfaceOptions_AddCategory(DMPOptions.panel);
	
--	DMPOptions.title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
--	DMPOptions.title:SetPoint("TOPLEFT", 16, -16)
--	DMPOptions.title:SetText("Darkmoon Professional")
	
--	DMPOptions.version = panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
--	DMPOptions.version:SetPoint("TOPLEFT", DMPOptions.title, "BOTTOMLEFT", 0, -8)
--	DMPOptions.version:SetText("Version "..GetAddOnMetadata("DarkmoonProfessional", "Version"))
	
--end

local function ClickFunc(self, button)
--	if button == "RightButton" then
--		InterfaceOptionsFrame_OpenToCategory("Darkmoon Professional");
--		InterfaceOptionsFrame_OpenToCategory("Darkmoon Professional");
		-- Called twice due to Blizzard bug
--	end
end

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("DarkmoonProfessional", {
	type = "launcher",
	icon = "Interface\\Icons\\INV_MISC_TICKET_DARKMOON_01",
	OnClick = ClickFunc
})

-- Faire Profession Quest Info
local questInfo = {
	[182] = {["questId"] = 29514}, -- Herbalism
	[186] = {["questId"] = 29518}, -- Mining
	[393] = {["questId"] = 29519}, -- Skinning
	[171] = {["questId"] = 29506}, -- Alchemy
	[164] = {["questId"] = 29508}, -- Blacksmithing
	[333] = {["questId"] = 29510}, -- Enchanting
	[202] = {["questId"] = 29511}, -- Engineering
	[773] = {["questId"] = 29515}, -- Inscription
	[755] = {["questId"] = 29516}, -- Jewelcrafting
	[165] = {["questId"] = 29517}, -- Leatherworking
	[197] = {["questId"] = 29520}, -- Tailoring
	[794] = {["questId"] = 29507}, -- Archaeology
	[185] = {["questId"] = 29509}, -- Cooking
	[356] = {["questId"] = 29513}, -- Fishing
	["Trophies"] = {["questId"] = 29433}, -- Kerri's Grisly Trophies
	["Moonfang"] = {["questId"] = 33354}, -- Den Mother's Demise
	["JFeasel"] = {["questId"] = 32175}, -- Jeremy Feasel's Battle Pet Daily
	["CFeasel"] = {["questId"] = 36471} -- Cristoph VonFeasel's Battle Pet Daily
}

-- string colors
local RED   = "ffFF0000"
local GREEN = "ff00FF00"
local YELGREEN  = "ff7FFF00"
local YELLOW  = "ffFFFF00"
local TEAL = "ff33ff99"
local YELPALE = "ffffff99"

local DARKMOON_EVENT_ID = 479
local DARKMOON_BELL_SOUND = 49563

-- Basic function to add colour to a string
local function AddColor(str,color)
	return "|c"..color..str.."|r"
end

-- check to see if the Faire is active
local function CheckDarkmoonOpen()
	local i
	local date = C_DateAndTime.GetCurrentCalendarTime()
	local today = date.monthDay
	local numDayEvents = C_Calendar.GetNumDayEvents(0,today)
	for i = 1,numDayEvents do
		local title = C_Calendar.GetDayEvent(0, today, i).eventID
		if title == DARKMOON_EVENT_ID then
			return true
		end
	end
	return false
end

local function GetItemName(itemId)
	local itemName = nil

	itemName, _, _, _, _, _, _, _ = GetItemInfo(itemId)

	if itemName == nil then
		return "Retreiving Item Name..."
	else
		return itemName
	end
end

local function GetQuestMatsText(skillId)
	local returnText
	local itemName
	local itemCount
	local itemMax
	
	if skillId == 171 then -- Alchemy
		itemName = GetItemName(1645)
		itemCount = GetItemCount(1645)
		itemMax = 5
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
		itemCount = GetItemCount(19299)
		itemMax = 5
		itemName = GetItemName(19299)
		returnText = returnText..", "..itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == 773 then -- Inscription
		itemName = GetItemName(39354)
		itemCount = GetItemCount(39354)
		itemMax = 5
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == 165 then -- Leatherworking
		itemName = GetItemName(2320)
		itemCount = GetItemCount(2320)
		itemMax = 5
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
		itemName = GetItemName(6260)
		itemCount = GetItemCount(6260)
		itemMax = 5
		returnText = returnText..", "..itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
		itemName = GetItemName(6529)
		itemCount = GetItemCount(6529)
		itemMax = 10
		returnText = returnText..", "..itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == 197 then -- Tailoring
		itemName = GetItemName(2320)
		itemCount = GetItemCount(2320)
		itemMax = 1
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
		itemName = GetItemName(6260)
		itemCount = GetItemCount(6260)
		itemMax = 1
		returnText = returnText..", "..itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
		itemName = GetItemName(2604)
		itemCount = GetItemCount(2604)
		itemMax = 1
		returnText = returnText..", "..itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == 794 then -- Archaeology
		archCurr = C_CurrencyInfo.GetCurrencyInfo(393)
		itemName = archCurr.name
		itemCount = archCurr.quantity
		itemMax = 15
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == 185 then -- Cooking
		itemName = GetItemName(30817)
		itemCount = GetItemCount(30817)
		itemMax = 5
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	elseif skillId == "Trophies" then -- Grisly Trophies
		local _, _, _, itemCount, itemMax = GetQuestObjectiveInfo(29433, 1, false);

		itemName = GetItemName(71096)
		returnText = itemName.." ("
		if itemCount < itemMax then
			returnText = returnText..AddColor(itemCount.."/"..itemMax, RED )
		else
			returnText = returnText..AddColor(itemCount.."/"..itemMax, GREEN )
		end
		returnText = returnText..")"
	else
		return ""
	end
	return ": "..returnText
end

local function GetQuestText(skillId)
	local returnText
	local questData = questInfo[skillId]
	local _, _, questFinished, _, _ = GetQuestObjectiveInfo(questData["questId"], 1, false);

	if C_QuestLog.IsQuestFlaggedCompleted(questData["questId"]) then -- Returns nil if not complete.
		returnText = AddColor("Completed", GREEN )
	elseif questFinished == true then -- True if the quest is ready to hand in.
		returnText = AddColor("Ready to Hand In", YELGREEN )
	elseif C_QuestLog.GetLogIndexForQuestID(questData["questId"]) ~= nil then -- nil if not in log, otherwise 
	returnText = AddColor("In Progress", YELLOW )
	else
		returnText = AddColor("Not Completed", RED )
	end
	
	return returnText
end

local function GetSkillText(key, index)
	local skillInfo = {}
	local returnTextL
	local returnTextR
	local questText
	local questMats = ""
	if index == nil then
		return key, AddColor("Not Learned", RED )
	else
		skillInfo["name"], _, skillInfo["rank"], skillInfo["maxRank"], _, _, skillInfo["skillLine"], _, _, _ = GetProfessionInfo(index)

		returnTextR = GetQuestText( skillInfo["skillLine"] )
		questMats = GetQuestMatsText( skillInfo["skillLine"] )

		returnTextL = skillInfo["name"].." ("..skillInfo["rank"].."/"..skillInfo["maxRank"]..")"..questMats

		return returnTextL, returnTextR
	end

end

function dataobj:OnTooltipShow()
	local skills = {}
	local skillorder = {"Profession 1", "Profession 2", "Archaeology", "Fishing", "Cooking"}
	local tmpLeft
	local tmpRight

	skills["Profession 1"], skills["Profession 2"], skills["Archaeology"], skills["Fishing"], skills["Cooking"] = GetProfessions()
	
	if CheckDarkmoonOpen() == true then
		self:AddLine("The Faire is currently "..AddColor("OPEN", GREEN ))
	else
		self:AddLine("The Faire is currently "..AddColor("CLOSED", RED ))
		--return
	end

	self:AddLine(" ") -- Just spacing it out a bit

	for _,value in ipairs(skillorder) do
		self:AddDoubleLine( GetSkillText( value, skills[ value ] ) )
	end

	self:AddLine(" ") -- Just spacing it out a bit

	tmpLeft = "Test Your Strength"..GetQuestMatsText( "Trophies" )
	tmpRight = GetQuestText( "Trophies" )
	self:AddDoubleLine( tmpLeft, tmpRight )

	tmpRight = GetQuestText( "Moonfang" )
	self:AddDoubleLine( "Den Mother's Demise", tmpRight )

	self:AddLine(" ") -- Just spacing it out a bit

	tmpRight = GetQuestText( "JFeasel" )
	self:AddDoubleLine( "Jeremy Feasel Pet Battle", tmpRight )

	tmpRight = GetQuestText( "CFeasel" )
	self:AddDoubleLine( "Cristoph VonFeasel Pet Battle", tmpRight )
	
--	self:AddLine(" ")
--	self:AddLine( AddColor( "Right Click for Options", YELPALE ))

end

function dataobj:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	GameTooltip:ClearLines()
	dataobj.OnTooltipShow(GameTooltip)
	GameTooltip:Show()
end

function dataobj:OnLeave()
	GameTooltip:Hide()
end


-- The GUI OnLoad function.
--
--function DarkmoonProfessional_Options_OnLoad(panel)
--	print("ARGH!")
	-- Set the Text for the Check boxes.
	--
--	DarkmoonProfessionalOptionsFrame_MinimapButton:SetText(DmPLoc.SHOW_MINIMAP);
--	print("Setting string"..DmPLoc.SHOW_MINIMAP)

    -- Set the name for the Category for the Panel
    --
--	panel.name = "Darkmoon Professional";

    -- When the player clicks okay, set the Saved Variables to the current Check Box setting
    --
--	panel.okay = function (self) DarkmoonProfessional_Close(); end;

    -- When the player clicks cancel, set the Check Box status to the Saved Variables.
--	panel.cancel = function (self)  DarkmoonProfessional_CancelOrLoad();  end;

    -- Add the panel to the Interface Options
    --
--	InterfaceOptions_AddCategory(panel);
--end




-- Locked and Loaded!
function DarkmoonProfessional_OnLoad(frame)
    -- We want to know when we're all loaded.
	frame:RegisterEvent("PLAYER_ENTERING_WORLD");
end

-- EVENTS be happenin'
function DarkmoonProfessional_OnEvent(frame, event, arg1, ...)
    
    if (event == "PLAYER_ENTERING_WORLD") then
        -- Initialise and set the Check Boxes in options from variables
		
        DarkmoonProfessional_Initialize();
--        DarkmoonProfessional_CancelOrLoad();
    end
end

-- Let's get this show on the road!
function DarkmoonProfessional_Initialize()
    -- Get Realm name and Check for validity
    realm = GetRealmName();

    if (darkmoonProfessional_init or (not realm)) then
        return;
    end

    -- initialise data structures
    if (darkmoonProfessional_config == nil) then
        darkmoonProfessional_config = { };
    end

    -- Set the default list if the Realm Variable does not exist
    if (darkmoonProfessional_config[realm] == nil) then
        darkmoonProfessional_config[realm] = { };
    end;


    -- Set variable to say we have been through this.
    DarkmoonProfessional_init = true;
	
	-- Mini Mappery
	if dataobj then
		if not ldbi:IsRegistered(addonName) then
			ldbi:Register(addonName, dataobj, darkmoonProfessional_minimap);
		end
	end

	if CheckDarkmoonOpen() then
		PlaySound( DARKMOON_BELL_SOUND )
	end
	
end


local function SlashCommands(msg, editbox)
	if msg == 'toggle' then
		darkmoonProfessional_minimap.show = not darkmoonProfessional_minimap.show
		if darkmoonProfessional_minimap.show then
			ldbi:Show(addonName)
		else
			ldbi:Hide(addonName)
		end
	elseif msg == 'show' then
		darkmoonProfessional_minimap.show = true
		ldbi:Show(addonName)
	elseif msg == 'hide' then
		darkmoonProfessional_minimap.show = false
		ldbi:Hide(addonName)
--	elseif msg == 'config' then
--		InterfaceOptionsFrame_OpenToCategory("Darkmoon Professional");
--		InterfaceOptionsFrame_OpenToCategory("Darkmoon Professional");
		-- Called twice due to Blizzard bug

	else 
		print(AddColor(addonName, TEAL)..": Arguments to "..AddColor("/dmp", YELPALE).." or "..AddColor("/darkmoonprofessional", YELPALE))
		print(AddColor("  show",YELPALE).." - Show the minimap icon")
		print(AddColor("  hide",YELPALE).." - Hide the minimap icon")
		print(AddColor("  toggle",YELPALE).." - Toggle show/hide the minimap icon")
--		print(AddColor("  config",YELPALE).." - Open the config window")
	end
end

_G['SLASH_' .. addonName .. 'minimaptoggle' .. 1] = '/dmp'
_G['SLASH_' .. addonName .. 'minimaptoggle' .. 2] = '/darkmoonprofessional'
SlashCmdList[addonName .. 'minimaptoggle'] = SlashCommands
--setupOptionsPanel()
